@@ -82,7 +82,7 @@ class Event:
             if e.any_data and (e.any_data.typecode().name) == "EventDetails":
                 details = e.any_data.value()
                 self.any_data = details.any_data
-                if details.host_application:
+                if self.source and details.host_application:
                     self.source.app = Accessible.makeAccessible(
                         details.host_application)
             else:
@@ -226,7 +226,10 @@ class EventListener(Accessibility__POA.EventListener):
         self.__registered = False
 
     def notifyEvent(self, event):
-        self.callback(event)
+        try:
+            self.callback(event)
+        except:
+            debug.printException(debug.LEVEL_WARNING)
 
     def __del__(self):
         self.deregister()
@@ -302,7 +305,11 @@ class KeystrokeListener(Accessibility__POA.DeviceEventListener):
 
         Returns True if the event has been consumed.
         """
-        return self.callback(event)
+        try:
+            return self.callback(event)
+        except:
+            debug.printException(debug.LEVEL_WARNING)
+            return False
 
     def __del__(self):
         self.deregister()
@@ -393,9 +400,9 @@ class Accessible:
         if Accessible._cache.has_key(e.source):
             obj = Accessible._cache[e.source]
             if obj.__dict__.has_key("name"):
-                del obj.name
+                del obj.__dict__["name"]
             if obj.__dict__.has_key("label"):
-                del obj.label
+                del obj.__dict__["label"]
 
     _onNameChanged = staticmethod(_onNameChanged)
 
@@ -410,9 +417,9 @@ class Accessible:
         if Accessible._cache.has_key(e.source):
             obj = Accessible._cache[e.source]
             if obj.__dict__.has_key("description"):
-                del obj.description
+                del obj.__dict__["description"]
             if obj.__dict__.has_key("label"):
-                del obj.label
+                del obj.__dict__["label"]
 
     _onDescriptionChanged = staticmethod(_onDescriptionChanged)
 
@@ -513,6 +520,8 @@ class Accessible:
 
         if obj.valid:
             Accessible._cache[acc] = obj
+        else:
+            obj = None
 
         return obj
 
@@ -550,7 +559,6 @@ class Accessible:
         # object or not.
         #
         self._acc = None
-        self.__origAcc = None
 
         # We'll also keep track of whether this object is any good to
         # us or not.  This object will be deleted when a defunct event
@@ -566,38 +574,50 @@ class Accessible:
         assert (not Accessible._cache.has_key(acc)), \
                "Attempt to create an Accessible that's already been made."
 
-        # The acc reference might be an Accessibility_Accessible or an
-        # Accessibility_Application, so try both.
+        # See if we have an application. Via bridges such as the Java
+        # access bridge, we might be getting a CORBA::Object, which is
+        # of little use to us.  We need to narrow it down to something
+        # we can use.  The first attempt is to see if we can get an
+        # application out of it.  Then we go for an accessible.
         #
+        self.accessible = None
         try:
-            self._acc = acc._narrow(Accessibility.Application)
+            self.accessible = acc._narrow(Accessibility.Application)
             try:
-                self.toolkitName = self._acc.toolkitName
+                self.toolkitName = application.toolkitName
             except:
                 self.toolkitName = None
             try:
-                self.version = self._acc.version
+                self.version = application.version
             except:
                 self.version = None
         except:
-            self._acc = acc._narrow(Accessibility.Accessible)
-
-        # [[[TODO: WDW - the AT-SPI appears to give us a different accessible
-        # when we repeatedly ask for the same child of a parent that manages
-        # its descendants.  So...we probably shouldn't cache those kind of
-        # children because we're likely to cause a memory leak. Logged as
-        # bugzilla bug 319675.]]]
-        #
-        # Save a reference to the AT-SPI object, and also save this
-        # new object away in the cache.
-        #
-        if self._acc:
-            self.__origAcc = acc
             try:
-                self._acc.ref()
+                self.accessible = acc._narrow(Accessibility.Accessible)
+            except:
+                debug.printException(debug.LEVEL_SEVERE)
+                debug.println(debug.LEVEL_SEVERE,
+                              "atspi.py:Accessible.__init__" \
+                              + " NOT GIVEN AN ACCESSIBLE!")
+                self.accessible = None
+
+        if self.accessible:
+            # [[[TODO: WDW - the AT-SPI appears to give us a
+            # different accessible when we repeatedly ask for the
+            # same child of a parent that manages its descendants.
+            # So...we probably shouldn't cache those kind of
+            # children because we're likely to cause a memory
+            # leak. Logged as bugzilla bug 319675.]]]
+            #
+            # Save a reference to the AT-SPI object, and also save this
+            # new object away in the cache.
+            #
+            try:
+                self.accessible.ref()
+                self._acc = acc
                 self.valid = True
             except:
-                self.valid = False
+                debug.printException(debug.LEVEL_SEVERE)
 
     def getStateString(self):
         """Returns a space-delimited string composed of the given object's
@@ -689,32 +709,43 @@ class Accessible:
         """Unrefs the AT-SPI Accessible associated with this object.
         """
 
-        if self._acc:
+        if self.accessible:
             try:
-                self._acc.unref()
+                self.accessible.unref()
             except:
                 pass
             try:
-                Accessible.deleteAccessible(self.__origAcc)
+                Accessible.deleteAccessible(self._acc)
+                self.accessible = None
+                self._acc = None
             except:
-                print(dir(self))
+                pass
 
     def __get_name(self):
         """Returns the object's accessible name as a string.
         """
 
-        name = self._acc.name
-        if settings.cacheValues:
+        # Combo boxes don't seem to issue accessible-name changed
+        # events, so we can't cache their names.  The main culprit
+        # here seems to be the combo box in gaim's "Join Chat" window.
+        #
+        name = self.accessible.name
+
+        if name and settings.cacheValues \
+            and (self.role != rolenames.ROLE_COMBO_BOX):
             self.name = name
+
         return name
 
     def __get_description(self):
         """Returns the object's accessible description as a string.
         """
 
-        description = self._acc.description
-        if settings.cacheValues:
-            self.description = self._acc.description
+        description = self.accessible.description
+
+        if description and settings.cacheValues:
+            self.description = description
+
         return description
 
     def __get_parent(self):
@@ -727,20 +758,23 @@ class Accessible:
         # get events for objects without a parent, but then the object ends
         # up getting a parent later on.
         #
-        obj = self._acc.parent
-        if not obj:
+        accParent = self.accessible.parent
+
+        if not accParent:
             return None
         else:
-            parent = Accessible.makeAccessible(obj);
+            parent = Accessible.makeAccessible(accParent);
+
             if settings.cacheValues:
                 self.parent = parent
+
             return parent;
 
     def __get_child_count(self):
         """Returns the number of children for this object.
         """
 
-        childCount = self._acc.childCount
+        childCount = self.accessible.childCount
 
         # We don't want to cache this value because it's possible that it
         # will continually change.
@@ -753,7 +787,7 @@ class Accessible:
         """Returns the index of this object in its parent's child list.
         """
 
-        index = self._acc.getIndexInParent()
+        index = self.accessible.getIndexInParent()
 
         # We don't want to cache this value because it's possible that it
         # will continually change.
@@ -772,7 +806,7 @@ class Accessible:
         has an example of this in its menus demo.
         """
 
-        role = self._acc.getRoleName()
+        role = self.accessible.getRoleName()
 
         # [[[WDW - HACK to coalesce menu items with children into
         # menus.  The menu demo in gtk-demo does this, and one
@@ -791,8 +825,9 @@ class Accessible:
             and (self.childCount > 0):
                 role = rolenames.ROLE_MENU
 
-        if settings.cacheValues:
+        if role and settings.cacheValues:
             self.role = role
+
         return role
 
     def __get_localized_rolename(self):
@@ -802,10 +837,11 @@ class Accessible:
         comparison.
         """
 
-        localizedRoleName = self._acc.getLocalizedRoleName()
+        localizedRoleName = self.accessible.getLocalizedRoleName()
 
-        if settings.cacheValues:
+        if localizedRoleName and settings.cacheValues:
             self.localizedRoleName = localizedRoleName
+
         return localizedRoleName
 
     def __get_state(self):
@@ -813,26 +849,34 @@ class Accessible:
         sequence of Accessible StateTypes.
         """
 
-        s = self._acc.getState()
-        s = s._narrow(Accessibility.StateSet)
-        state = s.getStates()
+        stateSet = self.accessible.getState()
+        if stateSet:
+            state = stateSet._narrow(Accessibility.StateSet).getStates()
+        else:
+            state = []
+
         # [[[WDW - we don't seem to always get appropriate state changed
         # information, so we will not cache state information.]]]
         #
-        #if settings.cacheValues:
+        #if state and settings.cacheValues:
         #    self.state = state
+
         return state
 
     def __get_relations(self):
         """Returns the Accessible RelationSet of this object as a list.
         """
 
-        relationSet = self._acc.getRelationSet()
         relations = []
+
+        relationSet = self.accessible.getRelationSet()
+
         for relation in relationSet:
             relations.append(relation._narrow(Accessibility.Relation))
+
         if settings.cacheValues:
             self.relations = relations
+
         return relations
 
     def __get_app(self):
@@ -875,8 +919,10 @@ class Accessible:
         debug.println(debug.LEVEL_FINEST, "Accessible app for %s is %s" \
                       % (self.accessibleNameToString(), \
                          obj.accessibleNameToString()))
+
         if settings.cacheValues:
             self.app = obj
+
         return obj
 
     def __get_extents(self, coordinateType = 0):
@@ -895,88 +941,24 @@ class Accessible:
         """
 
         component = self.component
+
         if not component:
             return None
-        else:
-            # [[[TODO: WDW - caching the extents is dangerous because
-            # the object may move, resulting in the current extents
-            # becoming way out of date.  Perhaps need to cache just
-            # the component interface and suffer the hit for getting
-            # the extents if we cannot figure out how to determine if
-            # the cached extents is out of date. Logged as bugzilla
-            # bug 319678.]]]
-            #
-            extents = component.getExtents(coordinateType)
-            #if settings.cacheValues:
-            #    self.extents = extents
-            return extents
 
-    def __get_label(self):
-        """Returns an object's label as a string.  The label is determined
-        using the following logic:
+        extents = component.getExtents(coordinateType)
 
-        1. If the object has a name, return the name
-
-        2. Else if the object has a LABELLED_BY relation, return the text of
-           the targets of this relation
-
-        3. Else if the object has no name, return the description
-
-        4. Else return an empty string
-        """
-
-        label = ""
-
-        # If the object has a name, just return the name.
+        # [[[TODO: WDW - caching the extents is dangerous because
+        # the object may move, resulting in the current extents
+        # becoming way out of date.  Perhaps need to cache just
+        # the component interface and suffer the hit for getting
+        # the extents if we cannot figure out how to determine if
+        # the cached extents is out of date. Logged as bugzilla
+        # bug 319678.]]]
         #
-        if self.name and (len(self.name) > 0):
-            label = self.name
+        #if settings.cacheValues:
+        #    self.extents = extents
 
-        # Does the object have a relation set?
-        #
-        if (len(label) == 0):
-            relations = self.relations
-            for relation in relations:
-                if relation.getRelationType() \
-                       == Accessibility.RELATION_LABELLED_BY:
-                    target = Accessible.makeAccessible(relation.getTarget(0))
-                    label = target.name
-                    break
-
-        # [[[WDW - HACK because push buttons can have labels as
-        # their children.  An example of the is is the Font: button on
-        # the General tab in the Editing Profile dialog in gnome-terminal.
-        # But, they can also be labelled by something.
-        # So...we'll make the label be a combination of the thing
-        # labelling them (above) plus their name or the combination of
-        # the names of their children if the children exist.]]]
-        #
-        if self.role == rolenames.ROLE_PUSH_BUTTON:
-            for i in range(0, self.childCount):
-                debug.println(debug.LEVEL_FINEST,
-                              "Accessible.__get_label looking at child %d" % i)
-                child = self.child(i)
-                if child.role == rolenames.ROLE_LABEL:
-                    if child.name and (len(child.name) > 0):
-                        if len(label) > 0:
-                            label += " " + child.name
-                        else:
-                            label = child.name
-
-        # If the object has no name, but has a description, return that
-        #
-        if (len(label) == 0) and self.description \
-                 and (len(self.description) > 0):
-            # [[[TODO: HACK because yelp actually goes through the trouble
-            # of setting the description of some of its text areas to
-            # "no description".]]]
-            #
-            if self.description != "no description":
-                label = self.description
-
-        if settings.cacheValues:
-            self.label = label
-        return label
+        return extents
 
     def __get_action(self):
         """Returns an object that implements the Accessibility_Action
@@ -984,11 +966,14 @@ class Accessible:
         the Accessibility_Action interface.
         """
 
-        action = self._acc.queryInterface("IDL:Accessibility/Action:1.0")
+        action = self.accessible.queryInterface("IDL:Accessibility/Action:1.0")
+
         if action:
             action = action._narrow(Accessibility.Action)
-        if settings.cacheValues:
+
+        if action and settings.cacheValues:
             self.action = action
+
         return action
 
     def __get_component(self):
@@ -997,12 +982,15 @@ class Accessible:
         the Accessibility_Component interface.
         """
 
-        component = self._acc.queryInterface(\
+        component = self.accessible.queryInterface(\
             "IDL:Accessibility/Component:1.0")
+
         if component:
             component = component._narrow(Accessibility.Component)
-        if settings.cacheValues:
+
+        if component and settings.cacheValues:
             self.component = component
+
         return component
 
     def __get_hypertext(self):
@@ -1011,12 +999,15 @@ class Accessible:
         the Accessibility_Hypertext interface.
         """
 
-        hypertext = self._acc.queryInterface(\
+        hypertext = self.accessible.queryInterface(\
             "IDL:Accessibility/Hypertext:1.0")
+
         if hypertext:
             hypertext = hypertext._narrow(Accessibility.Hypertext)
-        if settings.cacheValues:
+
+        if hypertext and settings.cacheValues:
             self.hypertext = hypertext
+
         return hypertext
 
     def __get_image(self):
@@ -1025,12 +1016,15 @@ class Accessible:
         the Accessibility_Image interface.
         """
 
-        image = self._acc.queryInterface(\
+        image = self.accessible.queryInterface(\
             "IDL:Accessibility/Image:1.0")
+
         if image:
             image = image._narrow(Accessibility.Image)
-        if settings.cacheValues:
+
+        if image and settings.cacheValues:
             self.image = image
+
         return image
 
     def __get_selection(self):
@@ -1039,12 +1033,15 @@ class Accessible:
         the Accessibility_Selection interface.
         """
 
-        selection = self._acc.queryInterface(\
+        selection = self.accessible.queryInterface(\
             "IDL:Accessibility/Selection:1.0")
+
         if selection:
             selection = selection._narrow(Accessibility.Selection)
-        if settings.cacheValues:
+
+        if selection and settings.cacheValues:
             self.selection = selection
+
         return selection
 
     def __get_table(self):
@@ -1053,11 +1050,14 @@ class Accessible:
         the Accessibility_Table interface.
         """
 
-        table = self._acc.queryInterface("IDL:Accessibility/Table:1.0")
+        table = self.accessible.queryInterface("IDL:Accessibility/Table:1.0")
+
         if table:
             table = table._narrow(Accessibility.Table)
-        if settings.cacheValues:
+
+        if table and settings.cacheValues:
             self.table = table
+
         return table
 
     def __get_text(self):
@@ -1066,11 +1066,14 @@ class Accessible:
         the Accessibility_Text interface.
         """
 
-        text = self._acc.queryInterface("IDL:Accessibility/Text:1.0")
+        text = self.accessible.queryInterface("IDL:Accessibility/Text:1.0")
+
         if text:
             text = text._narrow(Accessibility.Text)
-        if settings.cacheValues:
+
+        if text and settings.cacheValues:
             self.text = text
+
         return text
 
     def __get_value(self):
@@ -1079,11 +1082,14 @@ class Accessible:
         the Accessibility_Value interface.
         """
 
-        value = self._acc.queryInterface("IDL:Accessibility/Value:1.0")
+        value = self.accessible.queryInterface("IDL:Accessibility/Value:1.0")
+
         if value:
             value = value._narrow(Accessibility.Value)
-        if settings.cacheValues:
+
+        if value and settings.cacheValues:
             self.value = value
+
         return value
 
     def __getattr__(self, attr):
@@ -1101,8 +1107,6 @@ class Accessible:
 
         if attr == "name":
             return self.__get_name()
-        elif attr == "label":
-            return self.__get_label()
         elif attr == "description":
             return self.__get_description()
         elif attr == "parent":
@@ -1161,13 +1165,13 @@ class Accessible:
         #
         # Save away details we now know about this child
         #
-        acc = self._acc.getChildAtIndex(index)
-        if not acc:
-            return None
-        newChild = Accessible.makeAccessible(acc)
-        newChild.index = index
-        newChild.parent = self
-        newChild.app = self.app
+        newChild = None
+        accChild = self.accessible.getChildAtIndex(index)
+        if accChild:
+            newChild = Accessible.makeAccessible(accChild)
+            newChild.index = index
+            newChild.parent = self
+            newChild.app = self.app
         return newChild
 
     def toString(self, indent="", includeApp=True):
@@ -1209,10 +1213,6 @@ def getObjects(root):
     tree traversal, ignoring the children of objects which report the
     MANAGES_DESCENDANTS state is active.
 
-    NOTE: this will throw an InvalidObjectError exception if the
-    AT-SPI Accessibility_Accessible can no longer be reached via
-    CORBA.
-
     Arguments:
     - root: the Accessible object to traverse
 
@@ -1249,10 +1249,6 @@ def findByRole(root, role):
     getObjects and then deletes objects from the list that aren't of
     the specified role.  Logged as bugzilla bug 319740.]]]
 
-    NOTE: this will throw an InvalidObjectError exception if the
-    AT-SPI Accessibility_Accessible can no longer be reached via
-    CORBA.
-
     Arguments:
     - root the Accessible object to traverse
     - role the string describing the Accessible role of the object
@@ -1275,10 +1271,6 @@ def findByName(root, name):
     getObjects and then deletes objects from the list that don't have
     the specified name. Logged as bugzilla bug 319740.]]]
 
-    NOTE: this will throw an InvalidObjectError exception if the
-    AT-SPI Accessibility_Accessible can no longer be reached via
-    CORBA.
-
     Arguments:
     - root the Accessible object to traverse
     - name the string containing the Accessible name of the object
@@ -1363,10 +1355,6 @@ def getFrame(obj):
     """Returns the frame containing this object, or None if this object
     is not inside a frame.
 
-    NOTE: this will throw an InvalidObjectError exception if the
-    AT-SPI Accessibility_Accessible can no longer be reached via
-    CORBA.
-
     Arguments:
     - obj: the Accessible object
     """
@@ -1427,10 +1415,6 @@ def getNodeLevel(obj):
     relation, with 0 being the top level node.  If this object is
     not in a tree relation, then -1 will be returned.
 
-    NOTE: this will throw an InvalidObjectError exception if the
-    AT-SPI Accessibility_Accessible can no longer be reached via
-    CORBA.
-
     Arguments:
     -obj: the Accessible object
     """
@@ -1459,10 +1443,6 @@ def getAcceleratorAndShortcut(obj):
     """Gets the accelerator string (and possibly shortcut) for the given
     object.
 
-    NOTE: this will throw an InvalidObjectError exception if the
-    AT-SPI Accessibility_Accessible can no longer be reached via
-    CORBA.
-
     Arguments:
     - obj: the Accessible object
 
