@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -28,13 +28,12 @@ import math
 import atspi
 import braille
 import debug
-
 import orca
 import rolenames
 import settings
+import util
 
 from orca_i18n import _                     # for gettext support
-from rolenames import getBrailleForRoleName # localized role names
 
 class BrailleGenerator:
     """Takes accessible objects and produces a list of braille Regions
@@ -141,7 +140,7 @@ class BrailleGenerator:
         self.brailleGenerators[rolenames.ROLE_WINDOW]              = \
              self._getBrailleRegionsForWindow
 
-    def _getBrailleTextForAccelerator(self, obj):
+    def _getTextForAccelerator(self, obj):
         """Returns a string to be displayed that describes the keyboard
         accelerator (and possibly shortcut) for the given object.
 
@@ -151,25 +150,28 @@ class BrailleGenerator:
         Returns a string to be displayed.
         """
 
-        text = ""
+        if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            text = ""
+            
+            result = atspi.getAcceleratorAndShortcut(obj)
 
-        result = atspi.getAcceleratorAndShortcut(obj)
+            accelerator = result[0]
+            #shortcut = result[1]
 
-        accelerator = result[0]
-        #shortcut = result[1]
+            if len(accelerator) > 0:
+                text += "(" + accelerator + ")"
 
-        if len(accelerator) > 0:
-            text += "(" + accelerator + ")"
+            # [[[TODO: WDW - various stuff preserved while we work out the
+            # desired verbosity here.]]]
+            #
+            #if len(shortcut) > 0:
+            #    text += "(" + shortcut + ")"
 
-        # [[[TODO: WDW - various stuff preserved while we work out the
-        # desired verbosity here.]]]
-        #
-        #if len(shortcut) > 0:
-        #    text += "(" + shortcut + ")"
-
-        return text
-
-    def _getBrailleTextForAvailability(self, obj):
+            return text
+        else:
+            return None
+        
+    def _getTextForAvailability(self, obj):
         """Returns a string to be displayed that describes the availability
         of the given object.
 
@@ -179,12 +181,14 @@ class BrailleGenerator:
         Returns a string to be displayed.
         """
 
-        if obj.state.count(atspi.Accessibility.STATE_SENSITIVE):
-            return ""
-        else:
+        if (settings.brailleVerbosityLevel \
+            == settings.VERBOSITY_LEVEL_VERBOSE) \
+            and obj.state.count(atspi.Accessibility.STATE_SENSITIVE) == 0:
             return _("grayed")
-
-    def _getBrailleTextForValue(self, obj):
+        else:
+            return None
+        
+    def _getTextForValue(self, obj):
         """Returns the text to be displayed for the object's current value.
 
         Arguments:
@@ -232,6 +236,14 @@ class BrailleGenerator:
         #
         return valueString
 
+    def _getTextForRole(self, obj):
+        if (settings.brailleVerbosityLevel \
+            == settings.VERBOSITY_LEVEL_VERBOSE)\
+           and (obj.role != rolenames.ROLE_UNKNOWN):
+            return rolenames.getBrailleForRoleName(obj)
+        else:
+            return None
+
     def _debugGenerator(self, generatorName, obj):
         """Prints debug.LEVEL_FINER information regarding the braille
         generator.
@@ -264,23 +276,14 @@ class BrailleGenerator:
 
         self._debugGenerator("_getDefaultBrailleRegions", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
-
-        value = self._getBrailleTextForValue(obj)
-        if len(value) > 0:
-            if len(text):
-                text += " "
-            text += value
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if (obj.label == None) or (obj.role != rolenames.ROLE_UNKNOWN):
-                if len(text):
-                    text += " "
-                text += getBrailleForRoleName(obj)
+        regions = []
 
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForValue(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+        
         regions = []
         componentRegion = braille.Component(obj, text)
         regions.append(componentRegion)
@@ -314,20 +317,11 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForAnimation", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
-
-        if obj.description:
-            if len(text):
-                text += ": "
-            text += obj.description
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+        text = util.appendString(text, obj.description, ": ")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -366,20 +360,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckBox", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "<x>"
+            text = util.appendString(text, "<x>")
         else:
-            text += "< >"
+            text = util.appendString(text, "< >")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            text += " " + getBrailleForRoleName(obj)
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -400,27 +390,21 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckMenuItem", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "<x>"
+            text = util.appendString(text, "<x>")
         else:
-            text += "< >"
+            text = util.appendString(text, "< >")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                text += " " + getBrailleForRoleName(obj)
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForRole(obj))
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -455,68 +439,26 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForComboBox", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
         regions = []
-        regions.append(braille.Region(obj.label + " "))
 
-        # Find the text displayed in the combo box.  This is either:
-        #
-        # 1) The last text object that's a child of the combo box
-        # 2) The selected child of the combo box.
-        # 3) The contents of the text of the combo box itself when
-        #    treated as a text object.
-        #
-        # Preference is given to #1, if it exists.
-        #
-        # If the label of the combo box is the same as the utterance for
-        # the child object, then this utterance is only displayed once.
-        #
-        # [[[TODO: WDW - Combo boxes are complex beasts.  This algorithm
-        # needs serious work.  Logged as bugzilla bug 319745.]]]
-        #
-        textObj = None
-        for i in range(0, obj.childCount):
-            debug.println(debug.LEVEL_FINEST,
-                          "braillegenerator._getBrailleRegionsForComboBox " \
-                          + "looking at child %d" % i)
-            child = obj.child(i)
-            if child.role == rolenames.ROLE_TEXT:
-                textObj = child
+        focusedRegionIndex = 0
+        label = util.getDisplayedLabel(obj)
+        if label and (len(label) > 0):
+            regions.append(braille.Region(label + " "))
+            focusedRegionIndex = 1
 
-        if textObj:
-            result = atspi.getTextLineAtCaret(textObj)
-            line = result[0]
-            if line != obj.label:
-                regions.append(braille.Text(textObj))
-        else:
-            selectedItem = None
-            comboSelection = obj.selection
-            if comboSelection and comboSelection.nSelectedChildren > 0:
-                selectedItem = atspi.Accessible.makeAccessible(\
-                    comboSelection.getSelectedChild(0))
-            if selectedItem:
-                if selectedItem.label != obj.label:
-                    regions.append(braille.Region(selectedItem.label))
-            else:
-                if obj.text:
-                    result = atspi.getTextLineAtCaret(obj)
-                    selectedText = result[0]
-                    if selectedText != obj.label:
-                        regions.append(braille.Text(obj))
-                else:
-                    debug.println(
-                        debug.LEVEL_SEVERE,
-                        "ERROR: Could not find selected item for combo box.")
+        displayedText = util.getDisplayedText(obj)
+        if displayedText:
+            regions.append(braille.Region(displayedText))
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            regions.append(braille.Region(" " + getBrailleForRoleName(obj)))
+        if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            regions.append(braille.Region(
+                " " + rolenames.getBrailleForRoleName(obj)))
 
         # [[[TODO: WDW - perhaps if a text area was created, we should
         # give focus to it.]]]
         #
-        return [regions, regions[0]]
+        return [regions, regions[focusedRegionIndex]]
 
     def _getBrailleRegionsForDesktopIcon(self, obj):
         """Get the braille for a desktop icon.
@@ -617,22 +559,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForIcon", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.image:
             description = obj.image.imageDescription
             if len(description):
-                if len(text):
-                    text += " "
-                text += description
+                text = util.appendString(text, description)
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -698,23 +634,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForMenu", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, rolenames.getBrailleForRoleName(obj))
 
-        text = obj.label
-
-        if len(text):
-            text += " "
-        text += getBrailleForRoleName(obj)
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -748,23 +677,15 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForMenuItem", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                # [[[TODO - WDW comment this out for now until we work
-                # out verbosity.]]]
-                #
-                #text += " " + getBrailleForRoleName(obj)
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -786,17 +707,13 @@ class BrailleGenerator:
 
         regions = []
 
-        textRegion = braille.Text(obj, obj.label)
+        textRegion = braille.Text(obj, util.getDisplayedLabel(obj))
         regions.append(textRegion)
 
         eol = braille.Region(" $l")
         regions.append(eol)
 
         # We do not want the role at the end of text areas.
-        #
-        #if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-        #    text = " " + getBrailleForRoleName(obj)
-        #regions.append(braille.Region(text))
 
         return [regions, textRegion]
 
@@ -826,17 +743,15 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForPageTab", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
-        text = obj.label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                text += " " + getBrailleForRoleName(obj)
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -900,20 +815,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForRadioButton", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "&=y"
+            text = util.appendString(text, "&=y")
         else:
-            text += "& y"
+            text = util.appendString(text, "& y")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            text += " " + getBrailleForRoleName(obj)
+        text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -934,27 +845,20 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForRadioMenuItem", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        text = obj.label
-        if len(text):
-            text += " "
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
 
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text += "&=y"
+            text = util.appendString(text, "&=y")
         else:
-            text += "& y"
+            text = util.appendString(text, "& y")
 
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if obj == orca.locusOfFocus:
-                text += " " + getBrailleForRoleName(obj)
-                availability = self._getBrailleTextForAvailability(obj)
-                if len(availability) > 0:
-                    text += " " + availability + " "
-                accelerator = self._getBrailleTextForAccelerator(obj)
-                if len(accelerator) > 0:
-                    text += accelerator
+        if obj == orca.locusOfFocus:
+            text = util.appendString(text, self._getTextForAvailability(obj))
+            text = util.appendString(text,
+                                      self._getTextForAccelerator(obj),
+                                      "")
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -1004,9 +908,22 @@ class BrailleGenerator:
         and the second element is the Region which should get focus.
         """
 
-        self._debugGenerator("_getBrailleRegionsForProgressBar", obj)
+        self._debugGenerator("_getBrailleRegionsForSlider", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        regions = []
+
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        # Ignore the text on the slider.
+        #text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForValue(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+        
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
 
     def _getBrailleRegionsForSpinButton(self, obj):
         """Get the braille for a spin button.  If the object already has
@@ -1081,12 +998,10 @@ class BrailleGenerator:
                     regions = self._getBrailleRegionsForCheckBox(obj)
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
-                #elif action.getName(i) == "edit":
-                #    text = self._getSpeechForText(obj, True)
-                #    break
 
         if len(regions) == 0:
-            regions = self._getDefaultBrailleRegions(obj)
+            regions = self._getDefaultBrailleRegions(
+                util.getRealActiveDescendant(obj))
 
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
@@ -1144,13 +1059,10 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForTearOffMenuItem", obj)
 
-        text = getBrailleForRoleName(obj)
-
-        regions = []
-        componentRegion = braille.Component(obj, text)
-        regions.append(componentRegion)
-
-        return [regions, componentRegion]
+        componentRegion = braille.Component(
+            obj,
+            rolenames.getBrailleForRoleName(obj))
+        return [[componentRegion], componentRegion]
 
     def _getBrailleRegionsForTerminal(self, obj):
         """Get the braille for a terminal
@@ -1164,22 +1076,16 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForTerminal", obj)
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        label = None
+        title = None
         frame = atspi.getFrame(obj)
         if frame:
-            label = frame.name
-        if not label:
-            label = obj.label
-        text = label
-
-        if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
-            if len(text):
-                text += " "
-            text += getBrailleForRoleName(obj)
+            title = frame.name
+        if not title:
+            title = util.getDisplayedLabel(obj)
 
+        text = title
+        text = util.appendString(text, rolenames.getBrailleForRoleName(obj))
+        
         regions = []
         regions.append(braille.Region(text))
 
@@ -1319,7 +1225,7 @@ class BrailleGenerator:
                 # Sometimes, for some unknown reason, the child is None.
                 # We now test for this, rather than cause a traceback.
                 #
-                if child and child.role != rolenames.ROLE_SEPARATOR:
+                if child and (child.role != rolenames.ROLE_SEPARATOR):
 
                 # the following line has been removed because insensitive
                 # menu items can get focus in StarOffice.
@@ -1332,15 +1238,15 @@ class BrailleGenerator:
                     result = self.getBrailleRegions(child, False)
                     regions.extend(result[0])
 
-		    # This helps us determine which child is the selected
-		    # child.  Tracking the SELECTED state is not always
-		    # useful (it seems to be inconsistently handled by
-		    # toolkits), so we look at the parent's selection model.
-		    # In addition, we add a STATE_ARMED check here as a
-		    # workaround to the way OOo handles its menu items.
-		    #
+                    # This helps us determine which child is the selected
+                    # child.  Tracking the SELECTED state is not always
+                    # useful (it seems to be inconsistently handled by
+                    # toolkits), so we look at the parent's selection model.
+                    # In addition, we add a STATE_ARMED check here as a
+                    # workaround to the way OOo handles its menu items.
+                    #
                     if (selection and selection.isChildSelected(i)) \
-		       or child.state.count(atspi.Accessibility.STATE_ARMED):
+                       or child.state.count(atspi.Accessibility.STATE_ARMED):
                         selectedRegion = result[1]
 
         return [regions, selectedRegion]
@@ -1355,12 +1261,7 @@ class BrailleGenerator:
         Returns a list of Regions to display.
         """
 
-        verbosity = settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL,
-                                        settings.VERBOSITY_LEVEL_VERBOSE)
-
-        brailleRolenameStyle = settings.getSetting(
-            settings.BRAILLE_ROLENAME_STYLE,
-            settings.BRAILLE_ROLENAME_STYLE_LONG)
+        brailleRolenameStyle = settings.brailleRolenameStyle
 
         regions = []
         parent = obj.parent
@@ -1377,7 +1278,12 @@ class BrailleGenerator:
             #
             if (parent.role != rolenames.ROLE_FILLER) \
                 and (parent.role != rolenames.ROLE_SPLIT_PANE):
-                if (len(parent.label) > 0) \
+
+                # Announce the label and text of the object in the hierarchy.
+                #
+                label = util.getDisplayedLabel(parent)
+                text = util.getDisplayedText(parent)
+                if ((label and len(label)) or (text and len(text))) \
                     or (parent.role != rolenames.ROLE_PANEL):
                     regions.append(braille.Region(" "))
                     result = self.getBrailleRegions(parent, False)
@@ -1398,14 +1304,15 @@ class BrailleGenerator:
             row = parent.table.getRowAtIndex(obj.index)
             desc = parent.table.getRowDescription(row)
             if desc and len(desc):
-                if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
+                if settings.brailleRolenameStyle \
+                       == settings.VERBOSITY_LEVEL_VERBOSE:
                     if brailleRolenameStyle \
                            == settings.BRAILLE_ROLENAME_STYLE_LONG:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_ROW_HEADER].brailleLong
+                            rolenames.ROLE_ROW_HEADER].brailleLong + " "
                     else:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_ROW_HEADER].brailleShort
+                            rolenames.ROLE_ROW_HEADER].brailleShort + " "
                 else:
                     text = desc
                 regions.append(braille.Region(text))
@@ -1413,14 +1320,15 @@ class BrailleGenerator:
             col = parent.table.getColumnAtIndex(obj.index)
             desc = parent.table.getColumnDescription(col)
             if desc and len(desc):
-                if verbosity == settings.VERBOSITY_LEVEL_VERBOSE:
+                if settings.brailleVerbosityLevel \
+                       == settings.VERBOSITY_LEVEL_VERBOSE:
                     if brailleRolenameStyle \
                            == settings.BRAILLE_ROLENAME_STYLE_LONG:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_COLUMN_HEADER].brailleLong
+                            rolenames.ROLE_COLUMN_HEADER].brailleLong + " "
                     else:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_COLUMN_HEADER].brailleShort
+                            rolenames.ROLE_COLUMN_HEADER].brailleShort + " "
                 else:
                     text = desc
                 regions.append(braille.Region(text))
