@@ -101,6 +101,12 @@ class _Speaker(GNOME__POA.Speech.SpeechCallback):
 class SpeechServer(speechserver.SpeechServer):
     """Provides SpeechServer implementation for gnome-speech."""
 
+    # Dictionary of known running servers.  The key is the iid and the
+    # value is the SpeechServer instance.  We do this to enforce a
+    # singleton instance of any given server.
+    #
+    __activeServers = {}
+
     def __activateDriver(iid):
         driver = bonobo.activation.activate_from_id(iid,
                                                     0,
@@ -119,44 +125,58 @@ class SpeechServer(speechserver.SpeechServer):
 
     __activateDriver = staticmethod(__activateDriver)
 
+    def __createServer(iid):
+        server = None
+
+        if SpeechServer.__activeServers.has_key(iid):
+            server = SpeechServer.__activeServers[iid]
+        else:
+            driver = SpeechServer.__activateDriver(iid)
+            if driver:
+                server = SpeechServer(driver, iid)
+                SpeechServer.__activeServers[iid] = server
+
+        return server
+
+    __createServer = staticmethod(__createServer)
+
     def getFactoryName():
         """Returns a localized name describing this factory."""
         return _("GNOME Speech Services")
 
     getFactoryName = staticmethod(getFactoryName)
 
-    def getSpeechServerInfos():
-        """Enumerate available speech servers.
-
-        Returns a list of [name, id] values identifying the available
-        speech servers.  The name is a human consumable string and the
-        id is an object that can be used to create a speech server
-        via the getSpeechServer method.
+    def getSpeechServers():
+        """Gets available speech servers as a list.  The caller
+        is responsible for calling the shutdown() method of each
+        speech server returned.
         """
 
         # Get a list of all the drivers on the system and find out how many
         # of them work.
         #
-        servers = bonobo.activation.query(
+        knownServers = bonobo.activation.query(
             "repo_ids.has('IDL:GNOME/Speech/SynthesisDriver:0.3')")
 
-        speechServerInfos = []
+        for server in knownServers:
+            if not SpeechServer.__activeServers.has_key(server.iid):
+                try:
+                    SpeechServer.__createServer(server.iid)
+                except:
+                    debug.printException(debug.LEVEL_WARNING)
 
-        for server in servers:
-            try:
-                driver = SpeechServer.__activateDriver(server.iid)
-                if driver:
-                    speechServerInfos.append([driver.driverName, server.iid])
-            except:
-                debug.printException(debug.LEVEL_WARNING)
+        return SpeechServer.__activeServers.values()
 
-        return speechServerInfos
-
-    getSpeechServerInfos = staticmethod(getSpeechServerInfos)
+    getSpeechServers = staticmethod(getSpeechServers)
 
     def getSpeechServer(info=None):
+        """Gets a given SpeechServer based upon the info.
+        See SpeechServer.getInfo() for more info.
         """
-        """
+
+        if info and SpeechServer.__activeServers.has_key(info[1]):
+            return SpeechServer.__activeServers[info[1]]
+
         server = None
 
         gservers = bonobo.activation.query(
@@ -183,15 +203,11 @@ class SpeechServer(speechserver.SpeechServer):
             return None
 
         try:
-            driver = SpeechServer.__activateDriver(gserver.iid)
-            server = SpeechServer(driver, gserver.iid)
+            server = SpeechServer.__createServer(gserver.iid)
         except:
             for s in gservers:
                 try:
-                    driver = SpeechServer.__activateDriver(s.iid)
-                    if driver:
-                        server = SpeechServer(driver, s.iid)
-                        break
+                    server = SpeechServer.__createServer(s.iid)
                 except:
                     debug.printException(debug.LEVEL_WARNING)
                     pass
@@ -200,6 +216,15 @@ class SpeechServer(speechserver.SpeechServer):
 
     getSpeechServer = staticmethod(getSpeechServer)
 
+    def shutdownActiveServers():
+        """Cleans up and shuts down this factory.
+        """
+        for key in SpeechServer.__activeServers.keys():
+            server = SpeechServer.__activeServers[key]
+            server.shutdown()
+
+    shutdownActiveServers = staticmethod(shutdownActiveServers)
+
     def __init__(self, driver, iid):
         speechserver.SpeechServer.__init__(self)
         self.__speakers   = {}
@@ -215,17 +240,17 @@ class SpeechServer(speechserver.SpeechServer):
         """Gets the voice-independent ACSS rate value of a voice."""
 
         if not self.__rateInfo.has_key(speaker):
-            return 50
+            return 50.0
 
         [minRate, averageRate, maxRate] = self.__rateInfo[speaker]
         rate = speaker.getParameterValue("rate")
         if rate < averageRate:
-            return int(((rate - minRate) / (averageRate - minRate)) * 50)
+            return 50.0 * (rate - minRate) / (averageRate - minRate)
         elif rate > averageRate:
-            return 50 \
-                   + int(((rate - averageRate) / (maxRate - averageRate)) * 50)
+            return 50.0 \
+                   + (50.0 * (rate - averageRate) / (maxRate - averageRate))
         else:
-            return 50
+            return 50.0
 
     def __setRate(self, speaker, acssRate):
         """Determines the voice-specific rate setting for the
@@ -236,11 +261,11 @@ class SpeechServer(speechserver.SpeechServer):
             return
 
         [minRate, averageRate, maxRate] = self.__rateInfo[speaker]
-        if acssRate < 50:
-            rate = minRate + acssRate * (averageRate - minRate) / 50
-        elif acssRate > 50:
+        if acssRate < 50.0:
+            rate = minRate + acssRate * (averageRate - minRate) / 50.0
+        elif acssRate > 50.0:
             rate = averageRate \
-                   + (acssRate - 50) * (maxRate - averageRate) / 50
+                   + (acssRate - 50.0) * (maxRate - averageRate) / 50.0
         else:
             rate = averageRate
 
@@ -255,11 +280,11 @@ class SpeechServer(speechserver.SpeechServer):
             return
 
         [minPitch, averagePitch, maxPitch] = self.__pitchInfo[speaker]
-        if acssPitch < 5:
-            pitch = minPitch + acssPitch * (averagePitch - minPitch) / 5
-        elif acssPitch > 5:
+        if acssPitch < 5.0:
+            pitch = minPitch + acssPitch * (averagePitch - minPitch) / 5.0
+        elif acssPitch > 5.0:
             pitch = averagePitch \
-                    + (acssPitch - 5) * (maxPitch - averagePitch) / 5
+                    + (acssPitch - 5.0) * (maxPitch - averagePitch) / 5.0
         else:
             pitch = averagePitch
 
@@ -274,7 +299,7 @@ class SpeechServer(speechserver.SpeechServer):
             return
 
         [minVolume, averageVolume, maxVolume] = self.__volumeInfo[speaker]
-        volume = minVolume + acssGain * (maxVolume - minVolume) / 10
+        volume = minVolume + acssGain * (maxVolume - minVolume) / 10.0
 
         speaker.setParameterValue("volume", volume)
 
@@ -307,7 +332,10 @@ class SpeechServer(speechserver.SpeechServer):
                 break
 
         if not found:
-            return None
+            if len(voices) == 0:
+                return None
+            else:
+                voice = voices[0]
 
         s = self.__driver.createSpeaker(voice)
         speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
@@ -366,6 +394,10 @@ class SpeechServer(speechserver.SpeechServer):
                     try:
                         [self.__sayAll.currentContext, acss] = \
                             self.__sayAll.utteranceIterator.next()
+                        debug.println(debug.LEVEL_INFO,
+                                      "SPEECH OUTPUT: '" \
+                                      + self.__sayAll.currentContext.utterance\
+                                      + "'")
                         self.__sayAll.idForCurrentContext = self.__speak(
                             self.__sayAll.currentContext.utterance,
                             acss)
@@ -442,20 +474,21 @@ class SpeechServer(speechserver.SpeechServer):
         """
         self.speak(character, acss)
 
-    def speakUtterances(self, list, acss=None):
+    def speakUtterances(self, list, acss=None, interrupt=True):
         """Speaks the given list of utterances immediately.
 
         Arguments:
-        - list: list of strings to be spoken
-        - acss: acss.ACSS instance; if None,
-                the default voice settings will be used.
-                Otherwise, the acss settings will be
-                used to augment/override the default
-                voice settings.
+        - list:      list of strings to be spoken
+        - acss:      acss.ACSS instance; if None,
+                     the default voice settings will be used.
+                     Otherwise, the acss settings will be
+                     used to augment/override the default
+                     voice settings.
+        - interrupt: if True, stop any speech currently in progress.
         """
         i = 0
         for text in list:
-            self.speak(text, acss, i == 0)
+            self.speak(text, acss, interrupt and (i == 0))
             i += 1
 
     def __speak(self, text=None, acss=None, interrupt=True):
@@ -508,10 +541,6 @@ class SpeechServer(speechserver.SpeechServer):
         #
         text = text.replace("[", "[ ")
 
-        # Send the text to the GNOME Speech speaker
-        #
-        debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + text + "'")
-
         try:
             # [[[TODO: WDW - back this stop out for now.  The problem is
             # that we end up clipping too much speech, especially in the
@@ -544,9 +573,6 @@ class SpeechServer(speechserver.SpeechServer):
                      voice settings.
         - interrupt: if True, stops any speech in progress before
                      speaking the text
-
-        Returns an id of the thing being spoken or -1 if nothing is to
-        be spoken.
         """
         if self.__sayAll:
             self.stop()
@@ -564,6 +590,8 @@ class SpeechServer(speechserver.SpeechServer):
         """
         try:
             [context, acss] = utteranceIterator.next()
+            debug.println(debug.LEVEL_INFO,
+                          "SPEECH OUTPUT: '" + context.utterance + "'")
             self.__sayAll = _SayAll(utteranceIterator,
                                     context,
                                     self.__speak(context.utterance, acss),
@@ -641,15 +669,17 @@ class SpeechServer(speechserver.SpeechServer):
 
     def shutdown(self):
         """Shuts down the speech engine."""
-        for speaker in self.__speakers.values():
-            speaker.unref()
-        self.__speakers = {}
-        try:
-            self.__driver.unref()
-        except:
-            pass
-
-        self.__driver = None
+        if SpeechServer.__activeServers.has_key(self.__iid):
+            for speaker in self.__speakers.values():
+                speaker.stop()
+                speaker.unref()
+            self.__speakers = {}
+            try:
+                self.__driver.unref()
+            except:
+                pass
+            self.__driver = None
+            del SpeechServer.__activeServers[self.__iid]
 
     def reset(self, text=None, acss=None):
         """Resets the speech engine."""
