@@ -25,6 +25,11 @@ from orca_i18n import _ # for gettext support
 
 keynames = {}
 keynames["Shift_L"]      = _("left shift")
+keynames["Alt_L"]      = _("left alt")
+keynames["Control_L"]      = _("left control")
+keynames["Shift_L"]      = _("left shift")
+keynames["Alt_R"]      = _("right alt")
+keynames["Control_R"]      = _("right control")
 keynames["ISO_Left_Tab"] = _("left tab")
 keynames["SunF36"]       = _("F 11")
 keynames["SunF37"]       = _("F 12")
