@@ -17,9 +17,21 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-import gtk
-import os, signal, sys
+try:
+    # This can fail due to gtk not being available.  We want to
+    # be able to recover from that if possible.  The main driver
+    # for this is to allow "orca --text-setup" to work even if
+    # the desktop is not running.
+    #
+    import gtk
+except:
+    pass
+
+import getopt
+import os
+import signal
 import string
+import sys
 import time
 
 import atspi
@@ -28,6 +40,7 @@ import debug
 import keynames
 import keybindings
 import mag
+import platform
 import rolenames
 import settings
 import speech
@@ -39,7 +52,7 @@ from input_event import InputEventHandler
 
 from orca_i18n import _           # for gettext support
 
-# The user-settings module (see _loadUserSettings).
+# The user-settings module (see loadUserSettings).
 #
 _userSettings = None
 
@@ -521,7 +534,10 @@ def _isPrintableKey(event_string):
     Returns True if this is an alphanumeric or punctuation key.
     """
 
-    reply = event_string in string.printable
+    if event_string == "space":
+        reply = True
+    else:
+        reply = event_string in string.printable
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoPrintableKey: returning: %s" % reply)
     return reply
@@ -592,14 +608,18 @@ def _isActionKey(event_string):
     Returns True if this is an action key.
     """
 
-    actionKeys = [ "space", "Return", "Escape", "Tab", "BackSpace", "Delete",
-                   "Left", "Right", "Up", "Down", "Page_Up", "Page_Down" ]
+    actionKeys = [ "Return", "Escape", "Tab", "BackSpace", "Delete",
+                   "Page_Up", "Page_Down", "Home", "End" ]
 
     reply = event_string in actionKeys
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoActionKey: returning: %s" % reply)
     return reply
 
+# Records the last time a key was echoed.
+#
+lastKeyEchoTime = None
+
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -612,6 +632,14 @@ def _keyEcho(event):
     - event: an AT-SPI DeviceEvent
     """
 
+    global lastKeyEchoTime
+
+    # If this keyboard event was for an object like a password text
+    # field, then don't echo it.
+    #
+    if locusOfFocus and locusOfFocus.role == rolenames.ROLE_PASSWORD_TEXT:
+        return
+
     event_string = event.event_string
     debug.println(debug.LEVEL_FINEST,
                   "orca._keyEcho: string to echo: %s" % event_string)
@@ -624,6 +652,7 @@ def _keyEcho(event):
     # echoed.
     #
     if settings.enableKeyEcho:
+
         if _isPrintableKey(event_string):
             if not settings.enablePrintableKeys:
                 return
@@ -635,24 +664,28 @@ def _keyEcho(event):
             if not settings.enableModifierKeys:
                 return
 
-            # The control characters come through as control characters,
-            # so we just turn them into their ASCII equivalent.  NOTE that
-            # the upper case ASCII characters will be used (e.g., ctrl+a
-            # will be turned into the string "A").  All these checks here
-            # are to just do some sanity checking before doing the
-            # conversion. [[[WDW - this is making assumptions about
-            # mapping ASCII control characters to to UTF-8.]]]
-            #
-            if (event.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL)) \
-               and (not event.is_text) and (len(event_string) == 1):
-                value = ord(event.event_string[0])
-                if value < 32:
-                    event_string = chr(value + 0x40)
-
         elif _isLockingKey(event_string):
             if not settings.enableLockingKeys:
                 return
 
+            modifiers = event.modifiers
+            if event_string == _("Caps_Lock"):
+                if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
+                    event_string += " Off"
+                else:
+                    event_string += " On"
+            elif event_string == _("Num_Lock"):
+                # [[[TODO: richb - we are not getting a correct modifier
+                # state value returned when Num Lock is turned off.
+                # Commenting out the speaking of the bogus on/off state
+                # until this can be fixed.]]]
+                #
+                #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
+                #    event_string += " Off"
+                #else:
+                #    event_string += " On"
+                pass
+
         elif _isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
                 return
@@ -668,14 +701,17 @@ def _keyEcho(event):
 
         # Check to see if there are localized words to be spoken for
         # this key event.
-        try:
+        if event_string in keynames.keynames:
             event_string = keynames.keynames[event_string]
-        except:
-            debug.printException(debug.LEVEL_FINEST)
-            pass
 
         debug.println(debug.LEVEL_FINEST,
                       "orca._keyEcho: speaking: %s" % event_string)
+
+        # We keep track of the time as means to let others know that
+        # we are probably echoing a key and should not be interrupted.
+        #
+        lastKeyEchoTime = time.time()
+
         speech.speak(event_string, voice)
 
 def _processKeyboardEvent(event):
@@ -695,9 +731,11 @@ def _processKeyboardEvent(event):
     global lastInputEvent
     global _orcaModifierPressed
 
-    event_string = event.event_string
-
     # Log the keyboard event for future playback, if desired.
+    # Note here that the key event_string being output is
+    # exactly what we received.  The KeyboardEvent object,
+    # however, will translate the event_string for control
+    # characters to their upper case ASCII equivalent.
     #
     string = atspi.KeystrokeListener.keyEventToString(event)
     if _recordingKeystrokes and _keystrokesFile \
@@ -705,36 +743,41 @@ def _processKeyboardEvent(event):
         _keystrokesFile.write(string + "\n")
     debug.printInputEvent(debug.LEVEL_FINE, string)
 
+    keyboardEvent = KeyboardEvent(event)
+
+    # See if this is one of our special Orca modifiers (e.g., "Insert")
+    #
     orcaModifierKeys = settings.orcaModifierKeys
+    isOrcaModifier = orcaModifierKeys.count(keyboardEvent.event_string) > 0
 
     if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
         # Key presses always interrupt speech.
         #
         speech.stop()
 
-        _keyEcho(event)
+        # If learn mode is enabled, it will echo the keys.
+        #
+        if not settings.learnModeEnabled:
+            _keyEcho(keyboardEvent)
 
         # We treat the Insert key as a modifier - so just swallow it and
         # set our internal state.
         #
-        if orcaModifierKeys.count(event_string):
+        if isOrcaModifier:
             _orcaModifierPressed = True
-            return True
 
-    elif (event.type == atspi.Accessibility.KEY_RELEASED_EVENT) \
-         and orcaModifierKeys.count(event_string):
+    elif isOrcaModifier \
+        and (keyboardEvent.type == atspi.Accessibility.KEY_RELEASED_EVENT):
         _orcaModifierPressed = False
-        return True
+
+    if _orcaModifierPressed:
+        keyboardEvent.modifiers |= (1 << MODIFIER_ORCA)
 
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
     # The main reason this is staying as is is that we may not want
     # scripts to override fundamental Orca key bindings.]]]
     #
-    keyboardEvent = KeyboardEvent(event)
-    if _orcaModifierPressed:
-        keyboardEvent.modifiers |= (1 << MODIFIER_ORCA)
-
     consumed = False
     try:
         consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
@@ -742,11 +785,19 @@ def _processKeyboardEvent(event):
             consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
                        processKeyboardEvent(keyboardEvent)
         if (not consumed) and settings.learnModeEnabled:
-            if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
-                braille.displayMessage(event_string)
+            if keyboardEvent.type \
+                == atspi.Accessibility.KEY_PRESSED_EVENT:
+                # Check to see if there are localized words to be
+                # spoken for this key event.
+                #
+                braille.displayMessage(keyboardEvent.event_string)
+                event_string = keyboardEvent.event_string
+                if event_string in keynames.keynames:
+                    event_string = keynames.keynames[event_string]
                 speech.speak(event_string)
-            elif (event.type == atspi.Accessibility.KEY_RELEASED_EVENT) \
-                 and (event_string == "Escape"):
+            elif (keyboardEvent.type \
+                  == atspi.Accessibility.KEY_RELEASED_EVENT) \
+                 and (keyboardEvent.event_string == "Escape"):
                 exitLearnMode(None, keyboardEvent)
             consumed = True
     except:
@@ -754,7 +805,7 @@ def _processKeyboardEvent(event):
 
     lastInputEvent = keyboardEvent
 
-    return consumed
+    return consumed or isOrcaModifier
 
 ########################################################################
 #                                                                      #
@@ -879,7 +930,21 @@ def outlineAccessible(accessible, erasePrevious=True):
 #                                                                      #
 ########################################################################
 
-def _loadUserSettings(script=None, inputEvent=None):
+def _toggleSilenceSpeech(script=None, inputEvent=None):
+    """Toggle the silencing of speech.
+
+    Returns True to indicate the input event has been consumed.
+    """
+    speech.stop()
+    if settings.silenceSpeech:
+        settings.silenceSpeech = False
+        speech.speak(_("Speech enabled."))
+    else:
+        speech.speak(_("Speech disabled."))
+        settings.silenceSpeech = True
+    return True
+
+def loadUserSettings(script=None, inputEvent=None):
     """Loads (and reloads) the user settings module, reinitializing
     things such as speech if necessary.
 
@@ -900,12 +965,12 @@ def _loadUserSettings(script=None, inputEvent=None):
             reload(_userSettings)
             reloaded = True
         except ImportError:
-            debug.printException(debug.LEVEL_SEVERE)
+            debug.printException(debug.LEVEL_FINEST)
     else:
         try:
             _userSettings = __import__("user-settings")
         except ImportError:
-            debug.printException(debug.LEVEL_SEVERE)
+            debug.printException(debug.LEVEL_FINEST)
 
     if settings.enableSpeech:
         try:
@@ -915,6 +980,7 @@ def _loadUserSettings(script=None, inputEvent=None):
             debug.println(debug.LEVEL_CONFIGURATION,
                           "Speech module has been initialized.")
         except:
+            debug.printException(debug.LEVEL_SEVERE)
             debug.println(debug.LEVEL_SEVERE,
                           "Could not initialize connection to speech.")
     else:
@@ -925,6 +991,7 @@ def _loadUserSettings(script=None, inputEvent=None):
         try:
             braille.init(_processBrailleEvent, 7)
         except:
+            debug.printException(debug.LEVEL_WARNING)
             debug.println(debug.LEVEL_WARNING,
                           "Could not initialize connection to braille.")
 
@@ -943,6 +1010,44 @@ def _loadUserSettings(script=None, inputEvent=None):
 
     return True
 
+def _showPreferencesGUI(script=None, inputEvent=None):
+    """Displays the user interace to configure Orca and set up
+    user preferences using a GUI.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    try:
+        module = __import__(settings.guiPreferencesModule,
+                            globals(),
+                            locals(),
+                            [''])
+        module.showPreferencesUI()
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+        pass
+
+    return True
+
+def _showPreferencesConsole(script=None, inputEvent=None):
+    """Displays the user interace to configure Orca and set up
+    user preferences via a command line interface.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    try:
+        module = __import__(settings.consolePreferencesModule,
+                            globals(),
+                            locals(),
+                            [''])
+        module.showPreferencesUI()
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+        pass
+
+    return True
+
 # If True, this module has been initialized.
 #
 _initialized = False
@@ -1006,15 +1111,33 @@ def init(registry):
                                             0,
                                             keystrokeRecordingHandler))
 
+    preferencesSettingsHandler = InputEventHandler(\
+        _showPreferencesGUI,
+        _("Displays the preferences configuration dialog."))
+    _keyBindings.add(keybindings.KeyBinding(
+        "space", \
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
+        1 << MODIFIER_ORCA,
+        preferencesSettingsHandler))
+
     loadUserSettingsHandler = InputEventHandler(\
-        _loadUserSettings,
+        loadUserSettings,
         _("Reloads user settings and reinitializes services as necessary."))
     _keyBindings.add(keybindings.KeyBinding(
-        "s", \
+        "space", \
         (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
         (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
         loadUserSettingsHandler))
 
+    toggleSilenceSpeechHandler = InputEventHandler(\
+        _toggleSilenceSpeech,
+        _("Toggles the silencing of speech."))
+    _keyBindings.add(keybindings.KeyBinding(
+        "s", \
+        1 << MODIFIER_ORCA,
+        1 << MODIFIER_ORCA,
+        toggleSilenceSpeechHandler))
+
     listAppsHandler = InputEventHandler(
         printApps,
         _("Prints a debug listing of all known applications to the console where Orca is running."))
@@ -1073,7 +1196,7 @@ def init(registry):
     registry.registerEventListener(_onChildrenChanged,
                                    "object:children-changed:")
 
-    _loadUserSettings()
+    loadUserSettings()
 
     registry.registerKeystrokeListeners(_processKeyboardEvent)
 
@@ -1117,7 +1240,7 @@ def start(registry):
     registry.start()
 
 def abort():
-    sys.exit(1)
+    os._exit(1)
 
 def timeout():
     print "TIMEOUT: Hung while trying to shutdown.  Aborting."
@@ -1176,7 +1299,7 @@ exitCount = 0
 def shutdownOnSignal(signum, frame):
     global exitCount
 
-    print "Shutting down and exiting due to signal =", signum
+    #print "Shutting down and exiting due to signal =", signum
 
     # Well...we'll try to exit nicely, but if we keep getting called,
     # something bad is happening, so just quit.
@@ -1189,28 +1312,54 @@ def shutdownOnSignal(signum, frame):
     # Try to do a graceful shutdown if we can.
     #
     try:
-        shutdown()
-        return
+        if _initialized:
+            shutdown()
+            return
+        else:
+            # We always want to try to shutdown speech since the
+            # speech servers are very persistent about living.
+            #
+            speech.shutdown()
+            shutdown()
+            os._exit(0)
     except:
         abort()
 
 def abortOnSignal(signum, frame):
-    print "Aborting due to signal =", signum
+    #print "Aborting due to signal =", signum
     abort()
 
-def main():
-    userprefs = os.path.join(os.environ["HOME"], ".orca")
-    sys.path.insert(0, userprefs)
-    sys.path.insert(0, '') # current directory
+def usage():
+    """Prints out usage information."""
+    print _("Usage: orca [OPTION...]")
+    print
+    print _("-?, --help                   Show this help message")
+    print _("-v, --version                %s") % platform.version
+    print _("-s, --setup, --gui-setup     Set up user preferences")
+    print _("-t, --text-setup             Set up user preferences (text version)")
+    print _("-n, --no-setup               Skip set up of user preferences")
+    print
+    print _("If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used.")
+    print
+    print _("Report bugs to orca-list@gnome.org.")
+    pass
 
+def main():
     signal.signal(signal.SIGHUP, shutdownOnSignal)
     signal.signal(signal.SIGINT, shutdownOnSignal)
     signal.signal(signal.SIGTERM, shutdownOnSignal)
     signal.signal(signal.SIGQUIT, shutdownOnSignal)
     signal.signal(signal.SIGSEGV, abortOnSignal)
 
-    registry = atspi.Registry()
-    init(registry)
+    # See if the desktop is running.  If it is, the import of gtk will
+    # succeed.  If it isn't, the import will fail.
+    #
+    desktopRunning = False
+    try:
+        import gtk
+        desktopRunning = True
+    except:
+        pass
 
     # Do not run Orca if accessibility has not been enabled.
     #
@@ -1218,16 +1367,86 @@ def main():
     a11yEnabled = commands.getoutput(\
         "gconftool-2 --get /desktop/gnome/interface/accessibility")
     if a11yEnabled != "true":
-        message = _("Accessibility has not been enabled for this session.")
-        print message
-        speech.speak(message)
-        message = _("Please run orca-setup and then logout and log back in.")
-        print message
-        speech.speak(message)
-        braille.displayMessage(message)
+        _showPreferencesConsole()
         abort()
 
-    start(registry)
+    # Parse the command line options.
+    #
+    # Run the preferences setup if the user has specified
+    # "--setup" or "--text-setup" on the command line.  If the
+    # desktop is not running, we will fallback to the console-based
+    # method as appropriate.
+    #
+    bypassSetup     = False
+    setupRequested  = False
+    showGUI         = False
+    try:
+        # Note to translators: ? is for help
+        #                      h is for help
+        #                      s is for setup
+        #                      n is for no setup
+        #                      t is for text setup
+        #                      v is for version
+        #
+        opts, args = getopt.getopt(
+            sys.argv[1:],
+            _("?stnv"),
+            [_("help"),
+             _("setup"),
+             _("gui-setup"),
+             _("text-setup"),
+             _("no-setup"),
+             _("version")])
+        for opt, val in opts:
+            if opt in (_("-s"), _("--gui-setup"), _("--setup")):
+                setupRequested = True
+                showGUI = desktopRunning
+            if opt in (_("-t"), _("--text-setup")):
+                setupRequested = True
+                showGUI = False
+            if opt in (_("-n"), _("--no-setup")):
+                bypassSetup = True
+            if opt in (_("-?"), _("--help")):
+                usage()
+                os._exit(0)
+            if opt in (_("-v"), _("--version")):
+                print "Orca %s" % platform.version
+                os._exit(0)
+    except:
+        usage()
+        os._exit(2)
+
+    if setupRequested and (not bypassSetup) and (not showGUI):
+        _showPreferencesConsole()
+
+    if not desktopRunning:
+        print "Cannot start Orca because it cannot connect"
+        print "to the Desktop.  Please make sure the DISPLAY"
+        print "environment variable has been set."
+        return 1
+
+    userprefs = os.path.join(os.environ["HOME"], ".orca")
+    sys.path.insert(0, userprefs)
+    sys.path.insert(0, '') # current directory
+
+    registry = atspi.Registry()
+    init(registry)
+
+    # Check to see if the user wants the configuration GUI. It's
+    # done here so that the user's existing preferences can be used
+    # to set the initial GUI state.  We'll also force the set to
+    # be run if the preferences file doesn't exist, unless the
+    # user has bypassed any setup via the --no-setup switch.
+    #
+    if setupRequested and (not bypassSetup) and showGUI:
+        _showPreferencesGUI()
+    elif (not _userSettings) and (not bypassSetup):
+        if desktopRunning:
+            _showPreferencesGUI()
+        else:
+            _showPreferencesConsole()
+
+    start(registry) # waits until we stop the registry
     return 0
 
 if __name__ == "__main__":
