@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2005 Sun Microsystems Inc.
+# Copyright 2004-2006 Sun Microsystems Inc.
 # Copyright 2001, 2002 BAUM Retec, A.G.
 #
 # This library is free software; you can redistribute it and/or
@@ -18,8 +18,8 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""Provides a method, getRoleName, that converts the role name of an
-Accessible object into a localized string.
+"""Provides a methods that converts the role name of an Accessible
+object into localized strings for speech and braille.
 """
 
 import debug
@@ -235,7 +235,7 @@ rolenames[ROLE_DIRECTORY_PANE] = Rolename(ROLE_DIRECTORY_PANE,
 
 rolenames[ROLE_DRAWING_AREA] = Rolename(ROLE_DRAWING_AREA,
                                         _("draw"),
-                                        _("DrawingAarea"),
+                                        _("DrawingArea"),
                                         _("drawing area"))
 
 rolenames[ROLE_FILE_CHOOSER] = Rolename(ROLE_FILE_CHOOSER,
@@ -587,7 +587,11 @@ def getSpeechForRoleName(obj):
         return rolenames[name].speech
     else:
         debug.println(debug.LEVEL_WARNING, "No rolename for %s" % name)
-        return name
+        localizedRoleName = obj.localizedRoleName
+        if localizedRoleName and len(localizedRoleName):
+            return localizedRoleName
+        else:
+            return name
 
 def getShortBrailleForRoleName(obj):
     """Returns the localized name of the given Accessible object; the name is
@@ -606,7 +610,11 @@ def getShortBrailleForRoleName(obj):
         return rolenames[name].brailleShort
     else:
         debug.println(debug.LEVEL_WARNING, "No rolename for %s" % name)
-        return name
+        localizedRoleName = obj.localizedRoleName
+        if localizedRoleName and len(localizedRoleName):
+            return localizedRoleName
+        else:
+            return name
 
 def getLongBrailleForRoleName(obj):
     """Returns the localized name of the given Accessible object; the name is
@@ -625,7 +633,11 @@ def getLongBrailleForRoleName(obj):
         return rolenames[name].brailleLong
     else:
         debug.println(debug.LEVEL_WARNING, "No rolename for %s" % name)
-        return name
+        localizedRoleName = obj.localizedRoleName
+        if localizedRoleName and len(localizedRoleName):
+            return localizedRoleName
+        else:
+            return name
 
 def getBrailleForRoleName(obj):
     """Returns the localized name of the given Accessible object; the name is
@@ -640,24 +652,7 @@ def getBrailleForRoleName(obj):
     the 'brailleRolenameStyle' setting.
     """
 
-    brailleRolenameStyle = settings.getSetting(
-        settings.BRAILLE_ROLENAME_STYLE,
-        settings.BRAILLE_ROLENAME_STYLE_LONG)
-
-    if brailleRolenameStyle == settings.BRAILLE_ROLENAME_STYLE_SHORT:
+    if settings.brailleRolenameStyle == settings.BRAILLE_ROLENAME_STYLE_SHORT:
         return getShortBrailleForRoleName(obj)
     else:
         return getLongBrailleForRoleName(obj)
-
-def getRoleName(obj):
-    """Returns the localized name of the given Accessible object.
-    If a localized name cannot be discovered, this will return
-    the string as defined by the at-spi.
-
-    Arguments:
-    - obj: an Accessible object
-
-    Returns a string containing the localized name of the object.
-    """
-
-    return getSpeechForRoleName(obj)
