@@ -25,6 +25,8 @@ voices (identified by name) which can be combined with various
 attributes to create aural style sheets.
 """
 
+import debug
+
 class VoiceFamily(dict):
     """Holds the family description for a voice."""
 
@@ -48,11 +50,11 @@ class VoiceFamily(dict):
             self.update(props)
 
 class SayAllContext:
-    
+
     PROGRESS    = 0
     INTERRUPTED = 1
     COMPLETED   = 2
-    
+
     def __init__(self, obj, utterance, startOffset=-1, endOffset=-1):
         """Creates a new SayAllContext that will be passed to the
         SayAll callback handler for progress updates on speech.
@@ -61,7 +63,7 @@ class SayAllContext:
         If the object does have an accessible text specialization,
         then values >= 0 for startOffset and endOffset indicate
         where in the text the utterance has come from.
-        
+
         Arguments:
         -obj:         the Accessible being spoken
         -utterance:   the actual utterance being spoken
@@ -79,30 +81,35 @@ class SpeechServer:
     """Provides speech server abstraction."""
 
     def getFactoryName():
- 	"""Returns a localized name describing this factory."""
-	pass
+        """Returns a localized name describing this factory."""
+        pass
 
     getFactoryName = staticmethod(getFactoryName)
 
-    def getSpeechServerInfos():
-        """Enumerate available speech servers.
-
-        Returns a list of [name, id] values identifying the available
-        speech servers.  The name is a human consumable string and the
-	id is an object that can be used to create a speech server
-	via the getSpeechServer method.
+    def getSpeechServers():
+        """Gets available speech servers as a list.  The caller
+        is responsible for calling the shutdown() method of each
+        speech server returned.
         """
         pass
 
-    getSpeechServerInfos = staticmethod(getSpeechServerInfos)
+    getSpeechServers = staticmethod(getSpeechServers)
 
     def getSpeechServer(info):
         """Gets a given SpeechServer based upon the info.
+        See SpeechServer.getInfo() for more info.
         """
         pass
 
     getSpeechServer = staticmethod(getSpeechServer)
 
+    def shutdownActiveServers():
+        """Cleans up and shuts down this factory.
+        """
+        pass
+
+    shutdownActiveServers = staticmethod(shutdownActiveServers)
+
     def __init__(self):
         pass
 
@@ -122,10 +129,10 @@ class SpeechServer:
         Arguments:
         - text: text to be spoken
         - acss: acss.ACSS instance; if None,
-		the default voice settings will be used.
-		Otherwise, the acss settings will be
-		used to augment/override the default
-		voice settings.
+                the default voice settings will be used.
+                Otherwise, the acss settings will be
+                used to augment/override the default
+                voice settings.
 
         Output is produced by the next call to speak.
         """
@@ -151,37 +158,40 @@ class SpeechServer:
         Arguments:
         - character: text to be spoken
         - acss:      acss.ACSS instance; if None,
-		     the default voice settings will be used.
-		     Otherwise, the acss settings will be
-		     used to augment/override the default
-		     voice settings.
+                     the default voice settings will be used.
+                     Otherwise, the acss settings will be
+                     used to augment/override the default
+                     voice settings.
         """
         pass
 
-    def speakUtterances(self, list, acss=None):
+    def speakUtterances(self, list, acss=None, interrupt=True):
         """Speaks the given list of utterances immediately.
 
         Arguments:
-        - list: list of strings to be spoken
-        - acss: acss.ACSS instance; if None,
-		the default voice settings will be used.
-		Otherwise, the acss settings will be
-		used to augment/override the default
-		voice settings.
+        - list:      list of strings to be spoken
+        - acss:      acss.ACSS instance; if None,
+                     the default voice settings will be used.
+                     Otherwise, the acss settings will be
+                     used to augment/override the default
+                     voice settings.
+        - interrupt: if True, stop any speech currently in progress.
         """
         pass
 
-    def speak(self, text=None, acss=None):
+    def speak(self, text=None, acss=None, interrupt=True):
         """Speaks all queued text immediately.  If text is not None,
         it is added to the queue before speaking.
 
         Arguments:
-        - text: text to be spoken
-        - acss: acss.ACSS instance; if None,
-		the default voice settings will be used.
-		Otherwise, the acss settings will be
-		used to augment/override the default
-		voice settings.
+        - text:      optional text to add to the queue before speaking
+        - acss:      acss.ACSS instance; if None,
+                     the default voice settings will be used.
+                     Otherwise, the acss settings will be
+                     used to augment/override the default
+                     voice settings.
+        - interrupt: if True, stops any speech in progress before
+                     speaking the text
         """
         pass
 
@@ -200,8 +210,10 @@ class SpeechServer:
                              COMPLETED.
         """
         for [context, acss] in utteranceIterator:
+            debug.println(debug.LEVEL_INFO,
+                          "SPEECH OUTPUT: '" + context.utterance + "'")
             self.speak(context.utterance, acss)
-        
+
     def increaseSpeechRate(self, step=5):
         """Increases the speech rate.
         """
